﻿; *******************************************************************************************************************************************************
; AUTHOR        : MicrodevWeb
; PROJECT NAME  : RAD
; CLASS NAME    : TBL
; VERSION       : 1
; REQUIERED     : PB 5.60
; *******************************************************************************************************************************************************
; TODO resize le champs de recherce le combo

DeclareModule TBL
  ;======================================================================================================================================================
  ;-* PUBLIC CONSTANTE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  EnumerationBinary 
    #ShearchOn
    #FiltersOn
    #AlignToRight
    #AlignToCenter
    #CalculateColumn
  EndEnumeration 
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC INTERFACE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Interface _TBL
    AddColumn(ColumnName.s,Width.f,Flag.l=0)
    Draw()
    SetDoubleClickCallback(*CallBack)
    RefreshMe()
    GetIdSelected()
    SetOrder(Order.s)
    SetLigneSelected(LineSelected)
    Free()
    SetCalculateColumnCallBack(*callback)
    SetFiltersClause(Clause.s)
    FreeFiltersClause()
    SetSelectedCallback(*Callback)
  EndInterface
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC PROTOTYPES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Declare New(IdCanvas,TableName.s)
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
EndDeclareModule
Module TBL
  EnableExplicit
  UseSQLiteDatabase()
  UsePNGImageDecoder()
  UsePNGImageEncoder()
  Prototype.s CalculProto(id)
  ;======================================================================================================================================================
  ;-* PRIVATE CONSTANTE
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE STRUCTURES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Structure sColumn
    X.l
    W.l
    Name.s
    width.f
    flag.l
    IdBt.l
    BtPosX.l
    FiltersOn.b
    ShearchOn.b
    FilterClause.s
    IdFilters.l
    *CallculateCallBack
  EndStructure
  Structure sTable
    *VPROC
    TableName.s
    List myColumn.sColumn()
    Img.l
    IdCanvas.l
    ScrollH.l
    ScrollV.l
    ScrollVOn.l
    ColumnTitleHeight.l
    LineHeight.l
    List myId.l()
    LineSelected.l
    CurrentId.l
    *DoubleClickCallback
    *SelectedCallback
    Order.s
    CurrentColumn.l
    CurrentLine.l
    FiltersClause.s
  EndStructure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE VARIABLES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Global ColumnTitleFont=LoadFont(#PB_Any,"Arial",12,#PB_Font_HighQuality|#PB_Font_Bold)
  Global DataLineFont=LoadFont(#PB_Any,"Arial",11,#PB_Font_HighQuality)
  Global SelectedLineFont=LoadFont(#PB_Any,"Arial",11,#PB_Font_HighQuality|#PB_Font_Bold)
  Global img_loupe,img_FilterOf,img_FiltersOn
  Global gMouseX,gMouseY,*gCurrentColumn
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE PROTOTYPES
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Declare GetNumberVisibleLines(*This)
  Declare EventLine(*This)
  Declare DrawCanvas(*This)
  Declare RefreshMe(*This.sTable)
  Declare Resize(*This.sTable)
  Declare ResizeScrollH(*This.sTable)
  Declare Draw(*This)
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PRIVATE FUNCTIONS
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Procedure SelectTable()
    Protected *Tbl
    *Tbl=GetGadgetData(EventGadget())
    ProcedureReturn *Tbl
  EndProcedure
  Procedure EventScrollH()
    Protected *This.sTable=GetGadgetData(EventGadget())
    With *This
      DrawCanvas(*This)
    EndWith
  EndProcedure
  Procedure ManageScrollV(*This.sTable)
    Protected  idDb,req.s,nb,visibleLines
    With *This
      req="SELECT COUNT(*) FROM "+\TableName
      idDb=DB::Query(req)
      If Not idDb
        ProcedureReturn 
      EndIf
      FirstDatabaseRow(idDb)
      nb=GetDatabaseLong(idDb,0)
      CloseDatabase(idDb)
      ; Calcule du nombre de ligne visisible
      visibleLines=GetNumberVisibleLines(*This)
      If nb>visibleLines
        SetGadgetAttribute(\ScrollV,#PB_ScrollBar_Maximum,nb-visibleLines)
        \ScrollVOn=#True
        HideGadget(\ScrollV,#False)
      Else
        HideGadget(\ScrollV,#True)
        \ScrollVOn=#False
      EndIf
    EndWith
  EndProcedure
  Procedure ResizeScrollV(*This.sTable)
    Protected h,w
    With *This
      h=GadgetHeight(\IdCanvas)
      w=GadgetWidth(\IdCanvas)
      If IsGadget(\ScrollH)
        h-20
      EndIf
      ResizeGadget(\ScrollV,W-20,0,20,h)
    EndWith
  EndProcedure
  Procedure GetTotalWidth(*This.sTable)
    Protected w
    With *This
      ForEach \myColumn()
        w+GadgetWidth(\IdCanvas) * \myColumn()\width
      Next
      If w<GadgetWidth(\IdCanvas)
        w=GadgetWidth(\IdCanvas)
      EndIf
      ProcedureReturn w
    EndWith
  EndProcedure
  Procedure GetNumberVisibleLines(*This.sTable)
    Protected H,nb
    With *This
      H=GadgetHeight(\IdCanvas)-\ColumnTitleHeight
      If IsGadget(\ScrollH)
        H-20
      EndIf
      nb=Round(H/\LineHeight,#PB_Round_Nearest)
      ProcedureReturn nb
    EndWith
  EndProcedure
  Procedure.s BuiltFiltersClause(*This.sTable)
    Protected clause.s
    With *This
      ForEach \myColumn()
        If \myColumn()\flag & #ShearchOn
          If \myColumn()\FilterClause<>""
            If clause=""
              clause=" WHERE "
            Else
              clause+" AND "
            EndIf
            clause+\TableName+"."+\myColumn()\Name+" LIKE "+Chr(34)+\myColumn()\FilterClause+"%"+Chr(34)
            
          EndIf
        ElseIf \myColumn()\flag & #FiltersOn
          If \myColumn()\IdFilters<>-1
            If clause=""
              clause=" WHERE "
            Else
              clause+" AND "
            EndIf
            clause+\TableName+"."+\myColumn()\Name+" = "+Str(\myColumn()\IdFilters)
          EndIf
        EndIf
      Next
      ; Ajout du filtre général
      If \FiltersClause<>""
        If clause=""
          clause=" WHERE "
        Else
          clause+" AND "
        EndIf
        clause+\FiltersClause
      EndIf
      ProcedureReturn clause
    EndWith
  EndProcedure
  Procedure ManageFirstLine(*This.sTable)
    Protected idDb,req.s,nb
    With *This
      req="SELECT COUNT(*) FROM "+\TableName+BuiltFiltersClause(*This)
      idDb=DB::Query(req)
      If Not idDb
        ProcedureReturn 
      EndIf
      FirstDatabaseRow(idDb)
      nb=GetDatabaseLong(idDb,0)
      CloseDatabase(idDb)
      If nb>0
        If \LineSelected=-1 And nb>0
          \LineSelected=0
          SelectElement(\myId(),0)
          \CurrentId=\myId()
        Else
          If \LineSelected+1 > nb
            \LineSelected=0
            SelectElement(\myId(),0)
            \CurrentId=\myId()
          EndIf
        EndIf
      Else
        \LineSelected=-1
      EndIf
    EndWith
  EndProcedure
  Procedure DrawCanvas(*This.sTable)
    With *This
      Protected x,w,h,img
      ResizeScrollV(*This)
      StartVectorDrawing(CanvasVectorOutput(\IdCanvas))
      VectorSourceColor($FFD3D3D3)
      FillVectorOutput()
      w=GadgetWidth(\IdCanvas)
      h=ImageHeight(\Img)
      If IsGadget(\ScrollH)
        x=GetGadgetState(\ScrollH)
        h-20
      EndIf
      If \ScrollVOn
        w-20
      EndIf
      img=GrabImage(\Img,#PB_Any,x,0,w,h)
;       SaveImage(\Img,"Teste.png")
      MovePathCursor(0,0)
      DrawVectorImage(ImageID(img))
      FreeImage(img)
      StopVectorDrawing()
    EndWith
  EndProcedure
  Procedure DrawColumnTitle(*This.sTable)
    Protected H,x,y,xt,w,Txt.s,wg
    With *This
      H=\ColumnTitleHeight
      VectorSourceColor($FF808080)
      AddPathBox(0,0,ImageWidth(\Img),H)
      FillPath()
      VectorSourceColor($FFFFFFFF)
      VectorFont(FontID(ColumnTitleFont))
      Y=(H/2)-(VectorTextHeight("W")/2)
      wg=GadgetWidth(\IdCanvas)
      If \ScrollVOn
        wg-20
      EndIf
      ForEach \myColumn()
        w=wg * \myColumn()\width
        \myColumn()\W=w
        \myColumn()\X=x
        If \myColumn()\flag & #ShearchOn 
          MovePathCursor(x+(w-(\ColumnTitleHeight-1)),1)
          \myColumn()\BtPosX=x+(w-(\ColumnTitleHeight-1))
          DrawVectorImage(ImageID(img_loupe),255,\ColumnTitleHeight-2,\ColumnTitleHeight-2)
        EndIf
        If \myColumn()\flag & #FiltersOn
          MovePathCursor(x+(w-(\ColumnTitleHeight-1)),1)
          \myColumn()\BtPosX=x+(w-(\ColumnTitleHeight-1))
          If \myColumn()\FiltersOn
            DrawVectorImage(ImageID(img_FiltersOn),255,\ColumnTitleHeight-2,\ColumnTitleHeight-2)
          Else
            DrawVectorImage(ImageID(img_FilterOf),255,\ColumnTitleHeight-2,\ColumnTitleHeight-2)
          EndIf
        EndIf
        If Not \myColumn()\flag & #CalculateColumn
          Txt=DB::GetLabelColumn(\TableName,\myColumn()\Name)
        Else
          Txt=\myColumn()\Name
        EndIf
        While VectorTextWidth(Txt)>(w-8)
          Txt=Left(Txt,Len(Txt)-4)
          Txt+"..."
        Wend
        xt=x+(w/2)-(VectorTextWidth(Txt)/2)
        MovePathCursor(xt,y)
        DrawVectorText(Txt)
        x+w
      Next
    EndWith
  EndProcedure
  Procedure DrawColoredLine(*This.sTable)
    Protected H,N,i,y,W,LgnStart
    With *This
      y=\ColumnTitleHeight
      h=\LineHeight
      N=(ImageHeight(\Img)-y)/H
      W=ImageWidth(\Img)
      ; Si plus de lignes dans la table que de ligne affichée
      If \ScrollVOn
        LgnStart=GetGadgetState(\ScrollV)
      EndIf
      For i=LgnStart To N+LgnStart
        If i%2
          VectorSourceColor($FF00A5FF)
        Else
          VectorSourceColor($FFFFFFFF)
        EndIf
        AddPathBox(0,y,w,H)
        FillPath()
        y+H
      Next
    EndWith
  EndProcedure
  Procedure DrawDataLine(*This.sTable)
    Protected req.s,iddb,i,W,x,xt,y,yt,txt.s,h,n
    Protected NewList TableLinked.s()
    Protected NewList join.s()
    Protected myFunction.CalculProto
    ManageScrollV(*This)
    With *This
      y=\ColumnTitleHeight
      h=\LineHeight
      req="SELECT "+\TableName+"."+DB::GetIdTableName(\TableName)
      ForEach \myColumn()
        If i<ListSize(\myColumn())
          If Not \myColumn()\flag & #CalculateColumn
            req+","
          EndIf
        EndIf
        If Not \myColumn()\flag & #CalculateColumn
          Select DB::GetTypeOfColumn(\TableName,\myColumn()\Name)
            Case DB::#Type_Link
              req+DB::GetLinkedTable(\TableName,\myColumn()\Name)+"."+DB::GetColumnDisplayed(\TableName,\myColumn()\Name)
              AddElement(TableLinked())
              TableLinked()=DB::GetLinkedTable(\TableName,\myColumn()\Name)+
                            " ON "+DB::GetLinkedTable(\TableName,\myColumn()\Name)+"."+DB::GetLinkedColumn(\TableName,\myColumn()\Name)+
                            "="+\TableName+"."+\myColumn()\Name
            Default
              req+\TableName+"."+\myColumn()\Name
          EndSelect
        EndIf
      Next
      req+" FROM "+\TableName
      ForEach TableLinked()
        req+" LEFT JOIN "+TableLinked()
      Next
      req+BuiltFiltersClause(*This)
      req+\Order
      If \ScrollVOn
        req+" LIMIT "+Str(GetNumberVisibleLines(*This))+" OFFSET "+Str(GetGadgetState(\ScrollV))
      EndIf
      iddb=DB::Query(req)
      If Not iddb
        ProcedureReturn 
      EndIf
      ClearList(\myId())
      While NextDatabaseRow(iddb)
        AddElement(\myId())
        \myId()=GetDatabaseLong(iddb,0)
      Wend  
      CloseDatabase(iddb)
      ManageFirstLine(*This)
      iddb=DB::Query(req)
      If Not iddb
        ProcedureReturn 
      EndIf
      n=0
      While NextDatabaseRow(iddb)
        i=0
        SelectElement(\myId(),n)
        If \CurrentId=\myId()
          VectorSourceColor($FFFF0000)
          AddPathBox(0,y,ImageWidth(\Img),\LineHeight)
          FillPath()
        EndIf
        ForEach \myColumn()
          i+1
          xt=x+4
          w=GadgetWidth(\IdCanvas) * \myColumn()\width
          txt=""
          If Not \myColumn()\flag & #CalculateColumn
            Select DB::GetTypeOfColumn(\TableName,\myColumn()\Name)
              Case DB::#Type_Text,DB::#Type_Email,DB::#Type_Phone,DB::#Type_Note
                txt=GetDatabaseString(iddb,i)
              Case DB::#Type_Euro
                txt=FormatNumber(GetDatabaseFloat(iddb,i),2,"."," ")+" €"
              Case DB::#Type_Percentage
                txt=FormatNumber(GetDatabaseFloat(iddb,i),2,"."," ")+" %"
              Case DB::#Type_Link
                Select DB::GetTypeOfColumn(DB::GetLinkedTable(\TableName,\myColumn()\Name),DB::GetColumnDisplayed(\TableName,\myColumn()\Name))
                  Case DB::#Type_Text
                    txt=GetDatabaseString(iddb,i)
                  Case DB::#Type_Percentage
                    txt=FormatNumber(GetDatabaseFloat(iddb,i),2,"."," ")+" %"
                  Case DB::#Type_Euro
                    txt=FormatNumber(GetDatabaseFloat(iddb,i),2,"."," ")+" €"
                  Case DB::#Type_Date
                    txt=Right(GetDatabaseString(iddb,i),2)+"-"+
                        Mid(GetDatabaseString(iddb,i),5,2)+"-"+
                        Left(GetDatabaseString(iddb,i),4)
                EndSelect
              Case DB::#Type_Date
                txt=Right(GetDatabaseString(iddb,i),2)+"-"+
                    Mid(GetDatabaseString(iddb,i),5,2)+"-"+
                    Left(GetDatabaseString(iddb,i),4)
              Case DB::#Type_Integer
                txt=Str(GetDatabaseLong(iddb,i))
            EndSelect
          Else
            If \myColumn()\CallculateCallBack
              myFunction=\myColumn()\CallculateCallBack
              txt=myFunction(GetDatabaseLong(iddb,0))
            EndIf
          EndIf
          If \CurrentId=\myId()
            VectorFont(FontID(SelectedLineFont))
            VectorSourceColor($FFFFFFFF)
          Else
            VectorFont(FontID(DataLineFont))
            VectorSourceColor($FF000000)
          EndIf
          While VectorTextWidth(txt)>(w-8)
            txt=Left(txt,Len(txt )-4)
            txt+"..."
          Wend  
          yt=y+(h/2)-(VectorTextHeight(txt)/2)
          If \myColumn()\flag&#AlignToRight
            xt=(x+w-4)-VectorTextWidth(txt)
          EndIf
          If \myColumn()\flag&#AlignToCenter
            xt=(w/2)-(VectorTextWidth(txt)/2)
          EndIf
          MovePathCursor(xt,yt)
          DrawVectorText(txt)
          x+w
        Next
        x=0
        y+H
        n+1
      Wend
    EndWith
  EndProcedure
  Procedure DrawZone(*This.sTable)
    Protected n,x
    With *This
      VectorSourceColor($FF696969)
      AddPathBox(0,0,ImageWidth(\Img),ImageHeight(\Img))
      StrokePath(2)
      MovePathCursor(0,\ColumnTitleHeight)
      AddPathLine(ImageWidth(\Img),\ColumnTitleHeight)
      StrokePath(2)
      ForEach \myColumn()
        n+1
        x+\myColumn()\W
        If n<ListSize(\myColumn())
          MovePathCursor(X,0)
          AddPathLine(X,ImageHeight(\Img))
          StrokePath(2)
        EndIf
      Next
    EndWith
  EndProcedure
  Procedure DrawMyImage(*This.sTable)
    With *This
      StartVectorDrawing(ImageVectorOutput(\Img))
      VectorSourceColor($FFD3D3D3)
      FillVectorOutput()
      DrawColumnTitle(*This)
      DrawColoredLine(*This)
      DrawDataLine(*This)
      DrawZone(*This)
      StopVectorDrawing()
    EndWith
  EndProcedure
  Procedure EventScrollV()
    Protected *This.sTable=GetGadgetData(EventGadget())
    DrawMyImage(*This)
    DrawCanvas(*This)
  EndProcedure
  Procedure HoverColumnBt(*This.sTable)
    With *This
      \CurrentColumn=-1
      ; Si pas sur la ligne de titre de colonne
      If gMouseY>\ColumnTitleHeight
        ProcedureReturn #False
      EndIf
      ForEach \myColumn()
        If \myColumn()\flag & #ShearchOn Or \myColumn()\flag & #FiltersOn
          If gMouseX>=\myColumn()\BtPosX And gMouseX<=\myColumn()\BtPosX+(\ColumnTitleHeight-2)
            \CurrentColumn=ListIndex(\myColumn())
            ProcedureReturn #True
          EndIf
        EndIf
      Next
      ProcedureReturn #False
    EndWith
  EndProcedure
  Procedure HoverLine(*This.sTable)
    Protected H,n,gX
    With *This
      \CurrentLine=-1
      H=\ColumnTitleHeight+(GetNumberVisibleLines(*This) * \LineHeight)
      If gMouseY<\ColumnTitleHeight
        ProcedureReturn #False
      EndIf
      If gMouseY>H
        ProcedureReturn #False
      EndIf
      ; Calcul la ligne
      gX=gMouseY-\ColumnTitleHeight
      n=Round(gX / \LineHeight,#PB_Round_Down)
      \CurrentLine=n
      ProcedureReturn #True
    EndWith
  EndProcedure
  Procedure WhereIsMouse(*This.sTable)
    With *This
      If HoverColumnBt(*This)
        ProcedureReturn #True
      EndIf
      If HoverLine(*This)
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndWith
  EndProcedure
  Procedure FillCombo(*This.sTable)
    Protected req.s,idDb,n=1,IdSel
    With *This
      ClearGadgetItems(\myColumn()\IdBt)
      req="SELECT "+
          DB::GetLinkedColumn(\TableName,\myColumn()\Name)+","+
          DB::GetColumnDisplayed(\TableName,\myColumn()\Name)+
          " FROM "+DB::GetLinkedTable(\TableName,\myColumn()\Name)+
          " ORDER BY "+DB::GetColumnDisplayed(\TableName,\myColumn()\Name)
      AddGadgetItem(\myColumn()\IdBt,-1,"Pas de filtre")
      SetGadgetItemData(\myColumn()\IdBt,0,-1)
      idDb=DB::Query(req)
      If Not idDb
        If IsDatabase(idDb)
          CloseDatabase(idDb)
          ProcedureReturn 
        EndIf
      EndIf
      While NextDatabaseRow(idDb)
        AddGadgetItem(\myColumn()\IdBt,-1,GetDatabaseString(idDb,1))
        SetGadgetItemData(\myColumn()\IdBt,n,GetDatabaseLong(idDb,0))
        If \myColumn()\IdFilters=GetDatabaseLong(idDb,0)
          IdSel=n
        EndIf
        n+1
      Wend
      CloseDatabase(idDb)
      SetGadgetState(\myColumn()\IdBt,IdSel)
    EndWith
  EndProcedure
  Procedure EventStrShearch()
    Protected *This.sTable=GetGadgetData(EventGadget())
    With *This
      ChangeCurrentElement(\myColumn(),*gCurrentColumn)
      \myColumn()\FilterClause=GetGadgetText(\myColumn()\IdBt)
      SetGadgetState(\ScrollV,0)
      DrawMyImage(*This)
      DrawCanvas(*This )
      DrawCanvas(*This)
      If Not ListSize(\myId())
        \CurrentId=-1
      EndIf
      If \SelectedCallback
        CallFunctionFast(\SelectedCallback,\CurrentId)
      EndIf
    EndWith
  EndProcedure
  Procedure EventComboFilters()
    Protected *This.sTable=GetGadgetData(EventGadget())
    With *This
      ChangeCurrentElement(\myColumn(),*gCurrentColumn)
      \myColumn()\IdFilters=GetGadgetItemData(\myColumn()\IdBt,GetGadgetState(\myColumn()\IdBt))
      If \myColumn()\IdFilters<>-1
        \myColumn()\FiltersOn=#True
      Else
        \myColumn()\FiltersOn=#False
      EndIf
      HideGadget(\myColumn()\IdBt,#True)
      DrawMyImage(*This)
      DrawCanvas(*This)
      If Not ListSize(\myId())
        \CurrentId=-1
      EndIf
      If \SelectedCallback
        CallFunctionFast(\SelectedCallback,\CurrentId)
      EndIf
    EndWith
  EndProcedure
  Procedure EventBtColumn(*This.sTable)
    Protected x,y,w,h
    With *This
      Y=1
      h=\ColumnTitleHeight-2
      SelectElement(\myColumn(),\CurrentColumn)
      x=\myColumn()\X
      w=\myColumn()\W-\ColumnTitleHeight
      If \myColumn()\flag & #ShearchOn
        If \myColumn()\ShearchOn
          If IsGadget(\myColumn()\IdBt)
            SetGadgetText(\myColumn()\IdBt,"")
            HideGadget(\myColumn()\IdBt,#True)
          EndIf
          \myColumn()\ShearchOn=#False
          \myColumn()\FilterClause=""
          SetGadgetState(\ScrollV,0)
          DrawMyImage(*This)
          DrawCanvas(*This)
        Else
          If Not IsGadget(\myColumn()\IdBt)
            OpenGadgetList(\IdCanvas)
            \myColumn()\IdBt=StringGadget(#PB_Any,0,0,0,0,"")
            SetGadgetData(\myColumn()\IdBt,*This)
            BindGadgetEvent(\myColumn()\IdBt,@EventStrShearch())
            CloseGadgetList()
          EndIf
          \myColumn()\ShearchOn=#True
          ResizeGadget(\myColumn()\IdBt,x,y,w,h)
          HideGadget(\myColumn()\IdBt,#False)
          *gCurrentColumn=@\myColumn()
          SetActiveGadget(\myColumn()\IdBt)
          ProcedureReturn 
        EndIf
      EndIf
      If \myColumn()\flag & #FiltersOn
        If Not IsGadget(\myColumn()\IdBt)
          OpenGadgetList(\IdCanvas)
          \myColumn()\IdBt=ComboBoxGadget(#PB_Any,0,0,0,0)
          SetGadgetData(\myColumn()\IdBt,*This)
          BindGadgetEvent(\myColumn()\IdBt,@EventComboFilters())
          HideGadget(\myColumn()\IdBt,#True)
          CloseGadgetList()
        EndIf
        ResizeGadget(\myColumn()\IdBt,x,y,w,h)
        FillCombo(*This)
        *gCurrentColumn=@\myColumn()
        HideGadget(\myColumn()\IdBt,#False)
        SetActiveGadget(\myColumn()\IdBt)
      EndIf
    EndWith
  EndProcedure
  Procedure EventLine(*This.sTable)
    With *This
      \LineSelected=\CurrentLine
      If SelectElement(\myId(),\LineSelected)
        \CurrentId=\myId()
        If \SelectedCallback
          CallFunctionFast(\SelectedCallback,\CurrentId)
        EndIf
        RefreshMe(*This)
      EndIf
    EndWith
  EndProcedure
  Procedure EventCanvas()
    Protected min,max,value
    Protected *This.sTable=GetGadgetData(EventGadget())
    With *This
      Select EventType()
        Case #PB_EventType_MouseEnter
          SetActiveGadget(\IdCanvas)
        Case #PB_EventType_MouseMove
          gMouseX=GetGadgetAttribute(\IdCanvas,#PB_Canvas_MouseX)
          gMouseY=GetGadgetAttribute(\IdCanvas,#PB_Canvas_MouseY)
          If WhereIsMouse(*This)
            SetGadgetAttribute(\IdCanvas,#PB_Canvas_Cursor,#PB_Cursor_Hand)
          Else
            SetGadgetAttribute(\IdCanvas,#PB_Canvas_Cursor,#PB_Cursor_Default)
          EndIf
        Case #PB_EventType_LeftClick
          If \CurrentColumn<>-1
            EventBtColumn(*This)
          EndIf
          If \CurrentLine<>-1
            EventLine(*This)
          EndIf
        Case #PB_EventType_MouseWheel
          If \ScrollVOn
            max=GetGadgetAttribute(\ScrollV,#PB_ScrollBar_Maximum)
            min=GetGadgetAttribute(\ScrollV,#PB_ScrollBar_Minimum)
            value=GetGadgetState(\ScrollV)
            Select GetGadgetAttribute(\IdCanvas,#PB_Canvas_WheelDelta)
              Case -1
                If value<max
                  SetGadgetState(\ScrollV,value+1)
                  DrawMyImage(*This)
                  DrawCanvas(*This)
                EndIf
              Case 1
                If value>min
                  SetGadgetState(\ScrollV,value-1)
                  DrawMyImage(*This)
                  DrawCanvas(*This)
                EndIf
            EndSelect
          EndIf
        Case #PB_EventType_LeftDoubleClick
          If \DoubleClickCallback
            If \LineSelected<>-1
              CallFunctionFast(\DoubleClickCallback)
            EndIf
          EndIf
        Case  #PB_EventType_Resize
          Resize(*This)
        Case #PB_EventType_KeyDown
          max=GetGadgetAttribute(\ScrollV,#PB_ScrollBar_Maximum)
          min=GetGadgetAttribute(\ScrollV,#PB_ScrollBar_Minimum)
          value=GetGadgetState(\ScrollV)
          Select GetGadgetAttribute(\IdCanvas,#PB_Canvas_Key)
            Case #PB_Shortcut_Down
              If \LineSelected<>-1
                If \LineSelected<ListSize(\myId())-1
                  \LineSelected+1
                  SelectElement(\myId(),\LineSelected)
                  \CurrentId=\myId()
                  DrawMyImage(*This)
                  DrawCanvas(*This)
                Else
                  If \ScrollVOn And \LineSelected<max
                    SetGadgetState(\ScrollV,value+1)
                    DrawMyImage(*This)
                    LastElement(\myId())
                    \LineSelected=ListSize(\myId())-1
                    \CurrentId=\myId()
                    DrawMyImage(*This)
                    DrawCanvas(*This)
                  EndIf
                EndIf
              EndIf
            Case #PB_Shortcut_Up
              If \LineSelected<>-1
                If \LineSelected>0
                  \LineSelected-1
                  SelectElement(\myId(),\LineSelected)
                  \CurrentId=\myId()
                  DrawMyImage(*This)
                  DrawCanvas(*This)
                Else
                  If value>min
                    SetGadgetState(\ScrollV,value-1)
                    DrawMyImage(*This)
                    FirstElement(\myId())
                    \LineSelected=0
                    \CurrentId=\myId()
                    DrawMyImage(*This)
                    DrawCanvas(*This)
                  EndIf
                EndIf
              EndIf
          EndSelect
      EndSelect
    EndWith
  EndProcedure
  Procedure ResizeScrollH(*This.sTable)
    Protected h,w,wt
    With *This
      h=GadgetHeight(\IdCanvas)-20
      w=GadgetWidth(\IdCanvas)
      wt=GetTotalWidth(*This)
      If \ScrollVOn
        w-20
      EndIf
      If IsGadget(\ScrollH)
        ResizeGadget(\ScrollH,0,h,w,20)
        SetGadgetAttribute(\ScrollH,#PB_ScrollBar_Maximum,wt-w)
      EndIf
    EndWith
  EndProcedure
  Procedure Resize(*This.sTable)
    Protected w,h
    With *This
      ResizeGadget(\IdCanvas,#PB_Ignore,#PB_Ignore,#PB_Ignore,#PB_Ignore)
      ResizeScrollH(*This)
      ResizeScrollV(*This)
      w=GetTotalWidth(*This)
      
      h=GadgetHeight(\IdCanvas)
      If IsGadget(\ScrollH)
        h-20
      EndIf
      ResizeImage(\Img,w,h)
      DrawMyImage(*This)
      DrawCanvas(*This)
    EndWith
  EndProcedure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  ;======================================================================================================================================================
  ;-* PUBLIC FUNCTIONS
  ; -----------------------------------------------------------------------------------------------------------------------------------------------------
  Procedure New(IdCanvas,TableName.s)
    Protected *TBL.sTable=AllocateStructure(sTable)
    If Not IsImage(img_loupe)
      img_loupe=CatchImage(#PB_Any,?find_ico)
    EndIf
    If Not IsImage(img_FilterOf)
      img_FilterOf=CatchImage(#PB_Any,?filterOf)
    EndIf
    If Not IsImage(img_FiltersOn)
      img_FiltersOn=CatchImage(#PB_Any,?filterOn)
    EndIf
    With *TBL
      \TableName=TableName
      \IdCanvas=IdCanvas
      \ColumnTitleHeight=25
      \LineHeight=20
      \LineSelected=-1
      \VPROC=?VPROC
      SetGadgetData(\IdCanvas,*TBL)
      OpenGadgetList(\IdCanvas)
      \ScrollV=ScrollBarGadget(#PB_Any,0,0,0,0,0,0,10,#PB_ScrollBar_Vertical)
      SetGadgetData(\ScrollV,*TBL)
      CloseGadgetList()
      HideGadget(\ScrollV,#True)
      BindGadgetEvent(\ScrollV,@EventScrollV())
      ProcedureReturn *TBL
    EndWith
  EndProcedure
  Procedure AddColumn(*This.sTable,ColumnName.s,Width.f,Flag.l=0)
    With *This
      AddElement(\myColumn())
      \myColumn()\Name=ColumnName
      \myColumn()\flag=Flag
      \myColumn()\width=Width
      \myColumn()\FiltersOn=#False
      \myColumn()\IdFilters=-1
    EndWith
  EndProcedure
  Procedure Draw(*This.sTable)
    Protected w,h
    With *This
      If Not IsImage(\Img)
        w=GetTotalWidth(*This)
        h=GadgetHeight(\IdCanvas)
        If w>GadgetWidth(\IdCanvas)
          OpenGadgetList(\IdCanvas)
          \ScrollH=ScrollBarGadget(#PB_Any,0,h-20,GadgetWidth(\IdCanvas),20,0,w-GadgetWidth(\IdCanvas),10)
          BindGadgetEvent(\ScrollH,@EventScrollH())
          CloseGadgetList()
          h-20
        EndIf
        \Img=CreateImage(#PB_Any,w,h)
      EndIf
      BindGadgetEvent(\IdCanvas,@EventCanvas())
      DrawMyImage(*This)
      DrawCanvas(*This)
      If \SelectedCallback
        CallFunctionFast(\SelectedCallback,\CurrentId)
      EndIf
    EndWith
  EndProcedure
  Procedure RefreshMe(*This.sTable)
    DrawMyImage(*This)
    DrawCanvas(*This)
    With *This
      If *This\SelectedCallback
        CallFunctionFast(\SelectedCallback,\CurrentId)
      EndIf
    EndWith
  EndProcedure
  Procedure SetDoubleClickCallback(*This.sTable,*CallBack)
    With *This
      \DoubleClickCallback=*CallBack
    EndWith
  EndProcedure
  Procedure GetIdSelected(*This.sTable)
    With *This
      If \LineSelected=-1
        ProcedureReturn -1
      EndIf
      ProcedureReturn \CurrentId
    EndWith
  EndProcedure
  Procedure SetOrder(*This.sTable,Order.s)
    With *This
      \Order=Order
    EndWith
  EndProcedure
  Procedure SetLigneSelected(*This.sTable,LineSelected)
    With *This
      \LineSelected=LineSelected
    EndWith
  EndProcedure
  Procedure Free(*This.sTable)
    ClearStructure(*This,sTable)
  EndProcedure
  Procedure SetCalculateColumnCallBack(*This.sTable,*callback)
    With *This
      \myColumn()\CallculateCallBack=*callback
    EndWith
  EndProcedure
  Procedure SetFiltersClause(*This.sTable,Clause.s)
    With *This
      \FiltersClause=Clause
    EndWith
  EndProcedure
  Procedure FreeFiltersClause(*This.sTable)
    With *This
      \FiltersClause=""
    EndWith
  EndProcedure
  Procedure SetSelectedCallback(*This.sTable,*Callback)
    With *This
      \SelectedCallback=*Callback
    EndWith
  EndProcedure
  ;}-----------------------------------------------------------------------------------------------------------------------------------------------------
  DataSection
    VPROC:
    Data.i @AddColumn()
    Data.i @Draw()
    Data.i @SetDoubleClickCallback()
    Data.i @RefreshMe()
    Data.i @GetIdSelected()
    Data.i @SetOrder()
    Data.i @SetLigneSelected()
    Data.i @Free()
    Data.i @SetCalculateColumnCallBack()
    Data.i @SetFiltersClause()
    Data.i @FreeFiltersClause()
    Data.i @SetSelectedCallback()
  EndDataSection
  XIncludeFile "data.pbi"
EndModule
; IDE Options = PureBasic 5.60 (Windows - x64)
; CursorPosition = 893
; FirstLine = 73
; Folding = RgAAAAkBg
; Markers = 443
; EnableXP